local MAJOR_VERSION = "Race 1.2"
local MINOR_VERSION = tonumber(string.sub("$Revision: 1536 $", 12, -3))

if BabbleLib and BabbleLib.versions[MAJOR_VERSION] and BabbleLib.versions[MAJOR_VERSION].minor >= MINOR_VERSION then
	return
end

local locale = GetLocale and GetLocale() or "enUS"

local totalRaces = {
	["Human"] = "Human",
	["Night elf"] = "Night elf",
	["Dwarf"] = "Dwarf",
	["Gnome"] = "Gnome",
	
	["Orc"] = "Orc",
	["Tauren"] = "Tauren",
	["Troll"] = "Troll",
	["Undead"] = "Undead",
	
	["Humans"] = "Humans",
	["Night elves"] = "Night elves",
	["Dwarves"] = "Dwarves",
	["Gnomes"] = "Gnomes",
	
	["Orcs"] = "Orcs",
	["Tauren_PL"] = "Tauren",
	["Trolls"] = "Trolls",
	["Undead_PL"] = "Undead",
}

local englishToLocal

if locale == "koKR" then 
	englishToLocal = {
		["Human"] = "인간",
		["Night elf"] = "나이트 엘프",
		["Dwarf"] = "드워프",
		["Gnome"] = "노움",
		
		["Orc"] = "오크",
		["Tauren"] = "타우렌",
		["Troll"] = "트롤",
		["Undead"] = "언데드", 
				
		["Humans"] = "인간",
		["Night elves"] = "나이트 엘프",
		["Dwarves"] = "드워프",
		["Gnomes"] = "노움",
		
		["Orcs"] = "오크",
		["Tauren_PL"] = "타우렌",
		["Trolls"] = "트롤",
		["Undead_PL"] = "언데드",
	}
elseif locale == "deDE" then
	englishToLocal = {
		["Human"] = "Mensch",
		["Night elf"] = "Nachtelf",
		["Dwarf"] = "Zwerg",
		["Gnome"] = "Gnom",
		
		["Orc"] = "Orc",
		["Tauren"] = "Tauren",
		["Troll"] = "Troll",
		["Undead"] = "Untoter",
		
		["Humans"] = "Menschen",
		["Night elves"] = "Nachtelfen",
		["Dwarves"] = "Zwerge",
		["Gnomes"] = "Gnome",
		
		["Orcs"] = "Orcs",
		["Tauren_PL"] = "Tauren",
		["Trolls"] = "Trolle",
		["Undead_PL"] = "Untote",
	}
elseif locale == "frFR" then
	englishToLocal = {
		["Human"] = "Humain",
		["Night elf"] = "Elfe de la nuit",
		["Dwarf"] = "Nain",
		["Gnome"] = "Gnome",
		
		["Orc"] = "Orc",
		["Tauren"] = "Tauren",
		["Troll"] = "Troll",
		["Undead"] = "Mort-vivant",
		
		["Humans"] = "Humains",
		["Night elves"] = "Elfes de la nuit",
		["Dwarves"] = "Nains",
		["Gnomes"] = "Gnomes",
		
		["Orcs"] = "Orcs",
		["Tauren_PL"] = "Taurens",
		["Trolls"] = "Trolls",
		["Undead_PL"] = "Morts-vivants",
	}
elseif locale ~= "enUS" then
	-- no translations
	englishToLocal = {
	}
end

if englishToLocal then
	for key in pairs(englishToLocal) do
		if not totalRaces[key] then
			error("Improper translation exists. %q is likely misspelled for locale %s.", key, locale)
			break
		end
	end
end

-------------IRIEL'S-STUB-CODE--------------
local stub = {};

-- Instance replacement method, replace contents of old with that of new
function stub:ReplaceInstance(old, new)
   for k,v in pairs(old) do old[k]=nil; end
   for k,v in pairs(new) do old[k]=v; end
end

-- Get a new copy of the stub
function stub:NewStub()
  local newStub = {};
  self:ReplaceInstance(newStub, self);
  newStub.lastVersion = '';
  newStub.versions = {};
  return newStub;
end

-- Get instance version
function stub:GetInstance(version)
   if (not version) then version = self.lastVersion; end
   local versionData = self.versions[version];
   if (not versionData) then
	  message("Cannot find library instance with version '" 
			  .. version .. "'");
	  return;
   end
   return versionData.instance;
end

-- Register new instance
function stub:Register(newInstance)
   local version,minor = newInstance:GetLibraryVersion();
   self.lastVersion = version;
   local versionData = self.versions[version];
   if (not versionData) then
	  -- This one is new!
	  versionData = { instance = newInstance,
		 minor = minor,
		 old = {} 
	  };
	  self.versions[version] = versionData;
	  newInstance:LibActivate(self);
	  return newInstance;
   end
   if (minor <= versionData.minor) then
	  -- This one is already obsolete
	  if (newInstance.LibDiscard) then
		 newInstance:LibDiscard();
	  end
	  return versionData.instance;
   end
   -- This is an update
   local oldInstance = versionData.instance;
   local oldList = versionData.old;
   versionData.instance = newInstance;
   versionData.minor = minor;
   local skipCopy = newInstance:LibActivate(self, oldInstance, oldList);
   table.insert(oldList, oldInstance);
   if (not skipCopy) then
	  for i, old in ipairs(oldList) do
		 self:ReplaceInstance(old, newInstance);
	  end
   end
   return newInstance;
end

-- Bind stub to global scope if it's not already there
if (not BabbleLib) then
   BabbleLib = stub:NewStub();
end

-- Nil stub for garbage collection
stub = nil;
-----------END-IRIEL'S-STUB-CODE------------

local function assert(condition, message)
	if not condition then
		local stack = debugstack()
		local first = string.gsub(stack, "\n.*", "")
		local file = string.gsub(first, "^(.*\\.*).lua:%d+: .*", "%1")
		file = string.gsub(file, "([%(%)%.%*%+%-%[%]%?%^%$%%])", "%%%1")
		if not message then
			local _,_,second = string.find(stack, "\n(.-)\n")
			message = "assertion failed! " .. second
		end
		message = "BabbleLib-Race: " .. message
		local i = 1
		for s in string.gfind(stack, "\n([^\n]*)") do
			i = i + 1
			if not string.find(s, file .. "%.lua:%d+:") then
				error(message, i)
				return
			end
		end
		error(message, 2)
		return
	end
	return condition
end

local function argCheck(arg, num, kind, kind2, kind3, kind4)
	if tostring(type(arg)) ~= kind then
		if kind2 then
			if tostring(type(arg)) ~= kind2 then
				if kind3 then
					if tostring(type(arg)) ~= kind3 then
						if kind4 then
							if tostring(type(arg)) ~= kind4 then
								local _,_,func = string.find(debugstack(), "\n.-`(.-)'\n")
								assert(false, format("Bad argument #%d to `%s' (%s, %s, %s, or %s expected, got %s)", num, func, kind, kind2, kind3, kind4, type(arg)))
							end
						else
							local _,_,func = string.find(debugstack(), "\n.-`(.-)'\n")
							assert(false, format("Bad argument #%d to `%s' (%s, %s, or %s expected, got %s)", num, func, kind, kind2, kind3, type(arg)))
						end
					end
				else
					local _,_,func = string.find(debugstack(), "\n.-`(.-)'\n")
					assert(false, format("Bad argument #%d to `%s' (%s or %s expected, got %s)", num, func, kind, kind2, type(arg)))
				end
			end
		else
			local _,_,func = string.find(debugstack(), "\n.-`(.-)'\n")
			assert(false, format("Bad argument #%d to `%s' (%s expected, got %s)", num, func, kind, type(arg)))
		end
	end
end

local lib = {}
local localToEnglish

if locale == "enUS" then
	function lib:GetEnglish(race)
		argCheck(race, 2, "string")
		assert(localToEnglish[race], format("Race %q does not exist", race))
		return localToEnglish[race]
	end
	
	function lib:GetLocalized(race)
		argCheck(race, 2, "string")
		assert(totalRaces[race], format("Race %q does not exist", race))
		return totalRaces[race]
	end
	
	function lib:GetIterator()
		return pairs(totalRaces)
	end
	
	function lib:GetReverseIterator()
		return pairs(localToEnglish)
	end
	
	function lib:HasRace(race)
		argCheck(race, 2, "string")
		return (totalRaces[race] or localToEnglish[race]) and true or false
	end
else
	function lib:GetEnglish(race)
		argCheck(race, 2, "string")
		local t = localToEnglish[race]
		assert(t, format("Race %q does not exist or is not translated into %s", race, locale))
		return t
	end
	
	function lib:GetLocalized(race)
		argCheck(race, 2, "string")
		assert(totalRaces[race], format("Race %q does not exist", race))
		local t = englishToLocal[race]
		assert(t, format("Race %q is not translated into %s", race, locale))
		return t
	end
	
	local improperTranslation = nil
	for race in pairs(totalRaces) do
		if not englishToLocal[race] then
			improperTranslation = race
			break
		end
	end
	
	if improperTranslation then
		function lib:GetIterator()
			assert(false, "Race %q not translated into %s", improperTranslation, locale)
		end
		
		lib.GetReverseIterator = lib.GetIterator
	else
		function lib:GetIterator()
			return pairs(englishToLocal)
		end
		
		function lib:GetReverseIterator()
			return pairs(localToEnglish)
		end
	end
	
	function lib:HasRace(race)
		argCheck(race, 2, "string")
		return (totalRaces[race] or localToEnglish[race]) and true or false
	end
end

function lib:GetLibraryVersion()
	return MAJOR_VERSION, MINOR_VERSION
end

function lib:LibActivate(stub, oldLib, oldList)
	localToEnglish = {}
	for english, localized in pairs(englishToLocal or totalRaces) do
		localToEnglish[localized] = english
	end
	
	local mt = getmetatable(self) or {}
	mt.__call = self.GetLocalized
	setmetatable(self, mt)
end

function lib:LibDeactivate()
	totalRaces, localToEnglish, englishToLocal = nil
end

BabbleLib:Register(lib)
lib = nil
