local MAJOR_VERSION = "SpellTree 1.1"
local MINOR_VERSION = tonumber(string.sub("$Revision: 3860 $", 12, -3))

if BabbleLib and BabbleLib.versions[MAJOR_VERSION] and BabbleLib.versions[MAJOR_VERSION].minor >= MINOR_VERSION then
	return
end

local locale = GetLocale and GetLocale() or "enUS"

local totalTrees = {
	-- Druid
	["Balance"] = "Balance",
	["Feral Combat"] = "Feral Combat",
	["Restoration"] = "Restoration",
	-- Hunter
	["Beast Mastery"] = "Beast Mastery",
	["Marksmanship"] = "Marksmanship",
	["Survival"] = "Survival",
	-- Mage
	["Arcane"] = "Arcane",
	["Fire"] = "Fire",
	["Frost"] = "Frost",
	-- Paladin
	["Holy"] = "Holy",
	["Protection"] = "Protection",
	["Retribution"] = "Retribution",
	-- Priest
	["Discipline"] = "Discipline",
	["Holy"] = "Holy", -- same as Paladin
	["Shadow"] = "Shadow",
	-- Rogue
	["Assassination"] = "Assassination",
	["Combat"] = "Combat",
	["Subtlety"] = "Subtlety",
	-- Shaman
	["Elemental"] = "Elemental",
	["Enhancement"] = "Enhancement",
	["Restoration"] = "Restoration", -- same as Druid
	-- Warrior
	["Arms"] = "Arms",
	["Fury"] = "Fury",
	["Protection"] = "Protection", -- same as Paladin
	-- Warlock
	["Affliction"] = "Affliction",
	["Demonology"] = "Demonology",
	["Destruction"] = "Destruction",
}

local englishToLocal

if locale == "koKR" then 			
	englishToLocal = {
		-- Druid
		["Balance"] = "조화",
		["Feral Combat"] = "야생",
		["Restoration"] = "회복",
		-- Hunter
		["Beast Mastery"] = "야수",
		["Marksmanship"] = "사격",
		["Survival"] = "생존",		
		-- Mage
		["Arcane"] = "비전",
		["Fire"] = "화염",
		["Frost"] = "냉기",			
		-- Paladin
		["Holy"] = "신성",
		["Protection"] = "보호",
		["Retribution"] = "징벌",
		-- Priest
		["Discipline"] = "수양",
		["Holy"] = "신성", -- same as Paladin
		["Shadow"] = "암흑",		
		-- Rogue
		["Assassination"] = "암살",
		["Combat"] = "전투",
		["Subtlety"] = "잠행",
		-- Shaman
		["Elemental"] = "정기",
		["Enhancement"] = "고양",
		["Restoration"] = "복원", -- not same as Druid in Korean locale
		-- Warrior
		["Arms"] = "무기",
		["Fury"] = "분노",
		["Protection"] = "방어", -- not same as Paladin in Korean locale
		-- Warlock
		["Affliction"] = "고통",
		["Demonology"] = "악마",
		["Destruction"] = "파괴",
	}
elseif locale == "deDE" then
	englishToLocal = {
		-- Druid
		["Balance"] = "Gleichgewicht",
		["Feral Combat"] = "Wilder Kampf",
		["Restoration"] = "Wiederherstellung",
		-- Hunter
		["Beast Mastery"] = "Tierherrschaft",
		["Marksmanship"] = "Treffsicherheit",
		["Survival"] = "\195\156berleben",
		-- Mage
		["Arcane"] = "Arcan",
		["Fire"] = "Feuer",
		["Frost"] = "Frost",
		-- Paladin
		["Holy"] = "Heilig",
		["Protection"] = "Schutz",
		["Retribution"] = "Vergeltung",
		-- Priest
		["Discipline"] = "Disziplin",
		["Holy"] = "Heilig", -- same as Paladin
		["Shadow"] = "Schatten",
		-- Rogue
		["Assassination"] = "Meucheln",
		["Combat"] = "Kampf",
		["Subtlety"] = "T\195\164uschung",
		-- Shaman
		["Elemental"] = "Elementar",
		["Enhancement"] = "Verst\195\164rkung",
		["Restoration"] = "Wiederherstellung", -- same as Druid
		-- Warrior
		["Arms"] = "Waffen",
		["Fury"] = "Furor",
		["Protection"] = "Schutz", -- same as Paladin
		-- Warlock
		["Affliction"] = "Gebrechen",
		["Demonology"] = "D\195\164monologie",
		["Destruction"] = "Zerst\195\182rung",
	}
elseif locale == "frFR" then
	englishToLocal = {
		-- Druid
		["Balance"] = "Equilibre",
		["Feral Combat"] = "Combat sauvage",
		["Restoration"] = "Restauration",
		-- Hunter
		["Beast Mastery"] = "Ma\195\174trise des b\195\170tes",
		["Marksmanship"] = "Pr\195\169cision",
		["Survival"] = "Survie",
		-- Mage
		["Arcane"] = "Arcanes",
		["Fire"] = "Feu",
		["Frost"] = "Givre",
		-- Paladin
		["Holy"] = "Sacr\195\169",
		["Protection"] = "Protection",
		["Retribution"] = "Vindicte",
		-- Priest
		["Discipline"] = "Discipline",
		["Holy"] = "Sacr\195\169", -- same as Paladin
		["Shadow"] = "Ombre",
		-- Rogue
		["Assassination"] = "Assassinat",
		["Combat"] = "Combat",
		["Subtlety"] = "Finesse",
		-- Shaman
		["Elemental"] = "\195\137l\195\169mentaire",
		["Enhancement"] = "Am\195\169lioration",
		["Restoration"] = "Restauration", -- same as Druid
		-- Warrior
		["Arms"] = "Armes",
		["Fury"] = "Fureur",
		["Protection"] = "Protection", -- same as Paladin
		-- Warlock
		["Affliction"] = "Affliction",
		["Demonology"] = "D\195\169monologie",
		["Destruction"] = "Destruction",
	}
elseif locale ~= "enUS" then
	-- no translations
	englishToLocal = {
	}
end

if englishToLocal then
	for key in pairs(englishToLocal) do
		if not totalTrees[key] then
			error("Improper translation exists. %q is likely misspelled for locale %s.", key, locale)
			break
		end
	end
end

-------------IRIEL'S-STUB-CODE--------------
local stub = {};

-- Instance replacement method, replace contents of old with that of new
function stub:ReplaceInstance(old, new)
   for k,v in pairs(old) do old[k]=nil; end
   for k,v in pairs(new) do old[k]=v; end
end

-- Get a new copy of the stub
function stub:NewStub()
  local newStub = {};
  self:ReplaceInstance(newStub, self);
  newStub.lastVersion = '';
  newStub.versions = {};
  return newStub;
end

-- Get instance version
function stub:GetInstance(version)
   if (not version) then version = self.lastVersion; end
   local versionData = self.versions[version];
   if (not versionData) then
	  message("Cannot find library instance with version '" 
			  .. version .. "'");
	  return;
   end
   return versionData.instance;
end

-- Register new instance
function stub:Register(newInstance)
   local version,minor = newInstance:GetLibraryVersion();
   self.lastVersion = version;
   local versionData = self.versions[version];
   if (not versionData) then
	  -- This one is new!
	  versionData = { instance = newInstance,
		 minor = minor,
		 old = {} 
	  };
	  self.versions[version] = versionData;
	  newInstance:LibActivate(self);
	  return newInstance;
   end
   if (minor <= versionData.minor) then
	  -- This one is already obsolete
	  if (newInstance.LibDiscard) then
		 newInstance:LibDiscard();
	  end
	  return versionData.instance;
   end
   -- This is an update
   local oldInstance = versionData.instance;
   local oldList = versionData.old;
   versionData.instance = newInstance;
   versionData.minor = minor;
   local skipCopy = newInstance:LibActivate(self, oldInstance, oldList);
   table.insert(oldList, oldInstance);
   if (not skipCopy) then
	  for i, old in ipairs(oldList) do
		 self:ReplaceInstance(old, newInstance);
	  end
   end
   return newInstance;
end

-- Bind stub to global scope if it's not already there
if (not BabbleLib) then
   BabbleLib = stub:NewStub();
end

-- Nil stub for garbage collection
stub = nil;
-----------END-IRIEL'S-STUB-CODE------------
local function assert(condition, message)
	if not condition then
		local stack = debugstack()
		local first = string.gsub(stack, "\n.*", "")
		local file = string.gsub(first, "^(.*\\.*).lua:%d+: .*", "%1")
		file = string.gsub(file, "([%(%)%.%*%+%-%[%]%?%^%$%%])", "%%%1")
		if not message then
			local _,_,second = string.find(stack, "\n(.-)\n")
			message = "assertion failed! " .. second
		end
		message = "BabbleLib-SpellTree: " .. message
		local i = 1
		for s in string.gfind(stack, "\n([^\n]*)") do
			i = i + 1
			if not string.find(s, file .. "%.lua:%d+:") then
				error(message, i)
				return
			end
		end
		error(message, 2)
		return
	end
	return condition
end

local function argCheck(arg, num, kind, kind2, kind3, kind4)
	if tostring(type(arg)) ~= kind then
		if kind2 then
			if tostring(type(arg)) ~= kind2 then
				if kind3 then
					if tostring(type(arg)) ~= kind3 then
						if kind4 then
							if tostring(type(arg)) ~= kind4 then
								local _,_,func = string.find(debugstack(), "\n.-`(.-)'\n")
								assert(false, format("Bad argument #%d to `%s' (%s, %s, %s, or %s expected, got %s)", num, func, kind, kind2, kind3, kind4, type(arg)))
							end
						else
							local _,_,func = string.find(debugstack(), "\n.-`(.-)'\n")
							assert(false, format("Bad argument #%d to `%s' (%s, %s, or %s expected, got %s)", num, func, kind, kind2, kind3, type(arg)))
						end
					end
				else
					local _,_,func = string.find(debugstack(), "\n.-`(.-)'\n")
					assert(false, format("Bad argument #%d to `%s' (%s or %s expected, got %s)", num, func, kind, kind2, type(arg)))
				end
			end
		else
			local _,_,func = string.find(debugstack(), "\n.-`(.-)'\n")
			assert(false, format("Bad argument #%d to `%s' (%s expected, got %s)", num, func, kind, type(arg)))
		end
	end
end

local lib = {}
local localToEnglish

if locale == "enUS" then
	function lib:GetEnglish(tree)
		argCheck(tree, 2, "string")
		assert(totalTrees[tree], format("Spell Tree %q does not exist", tree))
		return tree
	end
	
	function lib:GetLocalized(tree)
		argCheck(tree, 2, "string")
		assert(totalTrees[tree], format("Spell Tree %q does not exist", tree))
		return tree
	end
	
	function lib:GetIterator()
		return pairs(totalTrees)
	end
	
	lib.GetReverseIterator = lib.GetIterator
	
	function lib:HasSpellTree(tree)
		argCheck(tree, 2, "string")
		return totalTrees[tree] and true or false
	end
else
	function lib:GetEnglish(tree)
		argCheck(tree, 2, "string")
		local t = localToEnglish[tree]
		assert(t, format("Spell Tree %q does not exist or is not translated into %s", tree, locale))
		return t
	end
	
	function lib:GetLocalized(tree)
		argCheck(tree, 2, "string")
		assert(totalTrees[tree], format("Spell Tree %q does not exist", tree))
		local t = englishToLocal[tree]
		assert(t, format("Spell Tree %q is not translated into %s", tree, locale))
		return t
	end
	
	local improperTranslation = nil
	for tree in pairs(totalTrees) do
		if not englishToLocal[tree] then
			improperTranslation = tree
			break
		end
	end
	
	if improperTranslation then
		function lib:GetIterator()
			assert(false, "Spell Tree %q not translated into %s", improperTranslation, locale)
		end
		
		lib.GetReverseIterator = lib.GetIterator
	else
		function lib:GetIterator()
			return pairs(englishToLocal)
		end
		
		function lib:GetReverseIterator()
			return pairs(localToEnglish)
		end
	end
	
	function lib:HasSpellTree(tree)
		argCheck(tree, 2, "string")
		return (totalTrees[tree] or localToEnglish[tree]) and true or false
	end
end

function lib:GetLibraryVersion()
	return MAJOR_VERSION, MINOR_VERSION
end

function lib:LibActivate(stub, oldLib, oldList)
	if locale ~= "enUS" then
		localToEnglish = {}
		for english, localized in pairs(englishToLocal) do
			localToEnglish[localized] = english
		end
	end
	
	local mt = getmetatable(self) or {}
	mt.__call = self.GetLocalized
	setmetatable(self, mt)
end

function lib:LibDeactivate()
	totalTrees, localToEnglish, englishToLocal = nil
end

BabbleLib:Register(lib)
lib = nil
